SuperStrict

Rem
	bbdoc: ErrorLogger
	about: This module provides a very simple to use Instance() which creates nicely formatted log-files.
End Rem
Module siliziumbrain.errorlogger
ModuleInfo "Version: v1.20"
ModuleInfo "Author: M0rgenstern - Siliziumbrain - Kevin Peter Hoffeld"
ModuleInfo "License: Public Domain"
ModuleInfo "History v1.00: 01 March 2012 - Final Release"
ModuleInfo "History v1.10: 14 July 2012 - MultiThreading and Singleton."
ModuleInfo "History v1.20: 15 July 2012 - Simple messages can now be logged if only information are needed but no error occured. Also: Fixed the HasFinishedWriting method. Also: Added possibility to log empty lines."

Import brl.threads
Import brl.Stream
Import siliziumbrain.uberobject
Import siliziumbrain.queue
Import brl.retro

Rem
	bbdoc: Error Logger to wirte error files.
	about:
	It allows to have %multiple Error-Log files (one for each instance). <br>
	It provides an simple way of logging nicely formatted errors.
Endrem
Type TErrorLogger Extends TObject
	Global _telErrorLogger:TErrorLogger

	Field _tllCurrentErrorLevel:TLogLevel
	Field _tqMessageQueue:TQueue
	Field _ttUpdateThread:TThread
	Field _tmMutex:TMutex
	Field _tcvCondVar:TCondVar
	Field _bIsFinished:Byte
	
	Rem
		bbdoc: Returns the instance of the TErrorLogger class.
		about: This is needed because the class is a singleton.
	endrem
	Function Instance:TErrorLogger()
		If(_telErrorLogger = Null) Then _telErrorLogger = New TErrorLogger
		
		Return _telErrorLogger
	End Function
	
	Rem
		bbdoc: Getter for the current error level.
		about: The current error level tells the logger which errors should be logged. <br>
		returns: The current error level (#TLogLevel).
	endrem
	Method CurrentErrorLevel:TLogLevel()
		Return _tllCurrentErrorLevel
	End Method
	
	Rem
		bbdoc: Setter for the current error level.
		about: Changes the current error level (#CurrentErrorLevel) to @piNewErrorLevel.
	endrem
	Method ChangeCurrentErrorLevel(piNewErrorLevel:Int)		
		CurrentErrorLevel().__SetLevel(piNewErrorLevel)
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Getter for the message queue.
	endrem
	Method __MessageQueue:TQueue()
		Return _tqMessageQueue
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Setter for the message queue.
	endrem
	Method __SetMessageQueue(ptQueue:TQueue)
		_tqMessageQueue = ptQueue
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Getter for the update thread.
	endrem
	Method __UpdateThread:TThread()
		Return _ttUpdateThread
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Setter for the update thread.
	endrem
	Method __SetUpdateThread(pttUpdateThread:TThread)
		_ttUpdateThread = pttUpdateThread
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Getter for the mutex (to use multithreading).
	endrem
	Method __Mutex:TMutex()
		Return _tmMutex
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Setter for the mutex.
	endrem
	Method __SetMutex(ptmMutex:TMutex)
		_tmMutex = ptmMutex
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Getter for the cond var (for multithreading).
	endrem
	Method __CondVar:TCondVar()
		Return _tcvCondVar
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Setter for the cond var.
	endrem
	Method __SetCondVar(ptcvCondVar:TCondVar)
		_tcvCondVar = ptcvCondVar
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Getter for the is finished value.
	endrem
	Method __IsFinished:Byte()
		Return _bIsFinished
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Setter for the is finished value.
	endrem
	Method __SetIsFinished(pbIsFinished:Byte)
		_bIsFinished = pbIsFinished
	End Method
	
	Rem
		bbdoc: Do NOT! use. Constructor.
		about: Constructor for an errorlogger. This should be only used by the class itself.
	endrem
	Method New()
		_tllCurrentErrorLevel = New TLogLevel.NewLogLevel(2)
		__SetMessageQueue(New TQueue)
		__SetIsFinished(False)
		__SetMutex(CreateMutex())
		__SetCondVar(CreateCondVar())
		__SetUpdateThread(CreateThread(Update, Object(Self)))
		__UpdateThread().Running()
	End Method
	
	Rem
		bbdoc: Logs an error.
		about: Use this method to log an error. <br>
		@psMessage is the error message self. <br>
		@psCategory is the category to which the error belongs. <br>
		@psComesFromFile is the filename where the error occurred. <br>
		@psFileToWriteIn is the path to the file where the error should be written into. <br>
		@piLogLevel is the error level of the logged error. <br>
		@poUsedObject can be an object which was used when an error occurred. If it is null, nothing about the object is written.
	endrem
	Method Log_Error(psMessage:String, psCategory:String, psComesFromFile:String, psFileToWriteIn:String, piLogLevel:Int, poUsedObject:Object = Null)
		'Only log error if the given level is high enough.
		If(piLogLevel >= CurrentErrorLevel().Level()) Then
			'Get the string of the object.
			Local psObjectString:String = ""
			If(poUsedObject <> Null) Then psObjectString = poUsedObject.ToString()
			
			'Repeat until the mutex can be locked.
			While(Not TryLockMutex(__Mutex()))
			
			Wend
				__SetIsFinished(False)
			
				'Add a new message to the queue.
				__MessageQueue().Enqueue(New TErrorMessage.NewLogMessage(psMessage, psCategory, psFileToWriteIn, psComesFromFile, piLogLevel, psObjectString))
				'Set the signal for the update thread.
				SignalCondVar(__CondVar())
			'Unlock the mutex.
			UnlockMutex(__Mutex())
		EndIf
	End Method
	
	Rem
		bbdoc: Logs a simple message.
		about: Use this method to log a simple message without many information. <br>
		@psMessage is the message self. <br>
		@psCategory is the category to which the message belongs.
		@psFileToWriteIn is the path to the file where the message should be written into. <br>
		@piLogLevel is the message level(<=> error level) of the logged message.
	endrem
	Method Log_Message(psMessage:String, psCategory:String, psFileToWriteIn:String, piLogLevel:Int)
		'Only log error if the given level is high enough.
		If(piLogLevel >= CurrentErrorLevel().Level()) Then
			'Repeat until the mutex can be locked.
			While(Not TryLockMutex(__Mutex()))
			
			Wend
				__SetIsFinished(False)
			
				'Add a new message to the queue.
				__MessageQueue().Enqueue(New TLogMessage.NewMessage(psMessage, psCategory, psFileToWriteIn, piLogLevel))
				'Set the signal for the update thread.
				SignalCondVar(__CondVar())
			'Unlock the mutex.
			UnlockMutex(__Mutex())
		EndIf
	End Method
	
	rem
		bbdoc: Just logs an empty line.
		about: Use this method if you want to have an empty line. <br>
		@psFileToWriteIn is the path to the file where the message should be written into. <br>
		@piLogLevel is the message level(<=> error level) of the logged message.
	endrem
	Method Log_NewLine(psFileToWriteIn:String, piLogLevel:Int)
		'Only log error if the given level is high enough.
		If(piLogLevel >= CurrentErrorLevel().Level()) Then
			While(Not TryLockMutex(__Mutex()))
			
			Wend
				__SetIsFinished(False)
			
				Local messg:TLogMessage = New TLogMessage.NewMessage("", "", psFileToWriteIn, piLogLevel)
				
				messg.__SetTimeWhenCreated("")
								
				'Add a new message to the queue.
				__MessageQueue().Enqueue(messg)
				'Set the signal for the update thread.
				SignalCondVar(__CondVar())
			'Unlock the mutex.
			UnlockMutex(__Mutex())
		EndIf
	End Method
	
	Rem
		bbdoc: Updates the error logger.
		about: This function runs in an own thread. <br>
		returns: Nothing.
	endrem
	Function Update:Object(ptelLogger:Object)
		'Run the function forever.
		Repeat
			'If there are messages in the queue, process them. Otherwise let the thread sleep.
			If(Instance().__MessageQueue().Count() > 0) Then
				'Lock the mutex.
				While(Not TryLockMutex(Instance().__Mutex()))
				
				Wend
					'For each currently available message in the queue.
					For Local index:Int = 0 Until Instance().__MessageQueue().Count()
						'Dequeue the message.
						Local MSG:TErrorMessage = TErrorMessage(Instance().__MessageQueue().Dequeue())
						'Get the file to write in.
						Local sFileToWriteIn:String = MSG.FileToWriteIn()
						Local strm:TStream
						'Decide if the file has to be created or just can be opened.
						If(FileType(sFileToWriteIn) = 1) Then
							strm = OpenFile(sFileToWriteIn)
							'Jump to the end of the file.
							strm.Seek(FileSize(sFileToWriteIn))
						Else
							strm = WriteFile(sFileToWriteIn)
						EndIf
						'Write the message in the file.
						strm.WriteLine(MSG.ToString())
						strm.Close()
					Next
					'Unlock the mutex.
				UnlockMutex(Instance().__Mutex())
			Else
				Instance().__SetIsFinished(True)
				'If there is no message in the queue, send the thread to sleep.
				WaitCondVar(Instance().__CondVar(), Instance().__Mutex())
			EndIf
		Forever
	End Function
	
	Rem
		bbdoc: Copies the object.
		about: Had to been implemented because of #TObject. <br>
		returns: Just the instance.
	End Rem
	Method Copy:TErrorLogger()
		Return Instance()
	End Method

	Rem
		bbdoc: Checks if two objects are equal
		about: Had to been implemented because of #TObject.
	End Rem
	Method Equals:Byte(ptoOtherObject:TObject)
		If(ptoOtherObject = Null) Then Return False
		
		If(ptoOtherObject = Self) Then Return True
		
		Return False
	End Method

	Rem
		bbdoc: Checks if the logger is still writing logs.
		about: This is needed to decide if a programm may end. <br>
		You have to wait until this returns true.
	endrem
	Method HasFinishedWriting:Byte()
		Return __IsFinished()
	End Method
End Type

Rem
	bbdoc: Class for the messages the errorlogger uses.
	about: The #TErrorLogger uses errormessages to log the errors.
endrem
Type TErrorMessage Extends TObject
	Field _sMessage:String
	Field _sTimeWhenCreated:String
	Field _sFileToWriteIn:String
	Field _sComesFromFile:String
	Field _sObjectString:String
	Field _sCategory:String
	Field _tllLogLevel:TLogLevel
	
	Rem
		bbdoc: Getter for the message.
		about: This method returns the given message. <br>
		returns: The given message.
	endrem
	Method Message:String()
		Return _sMessage
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Sets the given message to @psMessage.
	endrem
	Method __SetMessage(psMessage:String)
		_sMessage = psMessage
	End Method
	
	Rem
		bbdoc: Getter for the time when the message was created.
		about: This method returns the time when the message was created as a string. <br>
		returns: The time when the message was created.
	endrem
	Method TimeWhenCreated:String()
		Return _sTimeWhenCreated
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Sets the time when the message was created to @psTimeWhenCreated.
	endrem
	Method __SetTimeWhenCreated(psTimeWhenCreated:String)
		_sTimeWhenCreated = psTimeWhenCreated
	End Method
	
	Rem
		bbdoc: Getter for the path of the destination file.
		about: This method returns the path to the file where the message should be written in. <br>
		returns: The path of the destination file.
	endrem
	Method FileToWriteIn:String()
		Return _sFileToWriteIn
	End Method
	
	Rem
		bbdoc: Private method. Do not use.
		about: Sets the path of the destination file to @psFileToWriteIn.
	endrem
	Method __SetFileToWriteIn(psFileToWriteIn:String)
		_sFileToWriteIn = psFileToWriteIn
	End Method
	
	Rem
		bbdoc: Getter for the filename where the error occurred.
		about: This method returns the filename of the file where the error occured, so where the message comes from. <br>
		returns: Filename of the file where the message comes from.
	endrem
	Method ComesFromFile:String()
		Return _sComesFromFile
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Sets the filename where the message comes from to @psComesFromFile.
	endrem
	Method __SetComesFromFile(psComesFromFile:String)
		_sComesFromFile = psComesFromFile
	End Method
	
	Rem
		bbdoc: Getter for the object string.
		about: This method returns the string of the used object. <br>
		returns: The string of the object which was used.
	endrem
	Method ObjectString:String()
		Return _sObjectString
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Sets the object string to @psObjectString.
	endrem
	Method __SetObjectString(psObjectString:String)
		_sObjectString = psObjectString
	End Method
	
	Rem
		bbdoc: Getter for the category.
		about: This method returns the category of the message. <br>
		returns: The category of the message.
	endrem
	Method Category:String()
		Return _sCategory
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Sets the category of the message to @psCategory.
	endrem
	Method __SetCategory(psCategory:String)
		_sCategory = psCategory
	End Method
	
	Rem
		bbdoc: Getter for the loglevel.
		about: This method returns the loglevel (#TLogLevel) of the message. <br>
		returns: The loglevel of the message.
	endrem
	Method LogLevel:TLogLevel()
		Return _tllLogLevel
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Sets the loglevel of the message to @ptllLogLevel.
	endrem
	Method __SetLogLevel(ptllLogLevel:TLogLevel)
		_tllLogLevel = ptllLogLevel
	End Method
	
	Rem
		bbdoc: Constructor.
		about: Creates a new TErrorMessage. <br>
		@psMessage is the given message. <br>
		@psCategory is the category. <br>
		@psFileToWriteIn is the path of the destination file. <br>
		@psComesFromFile is the filename of the file message comes from. <br>
		@piLogLevel is the loglevel. <br>
		@psObjectString is the string of the used object.
	endrem
	Method NewLogMessage:TErrorMessage(psMessage:String, psCategory:String, psFileToWriteIn:String, psComesFromFile:String, piLogLevel:Int, psObjectString:String = "")
		__SetTimeWhenCreated(CurrentDate() + " - " + CurrentTime())
		__SetCategory(psCategory)
		__SetMessage(psMessage)
		__SetComesFromFile(psComesFromFile)
		__SetFileToWriteIn(psFileToWriteIn)
		
		If(Trim(psObjectString) = "") Then psObjectString = "No Object used."
		__SetObjectString(psObjectString)
		__SetLogLevel(New TLogLevel.NewLogLevel(piLogLevel))
		
		Return Self
	End Method
	
	Rem
		bbdoc:Copies the object.
		about: Had to been implemented because of #TObject.
	End Rem
	Method Copy:TErrorMessage()
		Return New TErrorMessage.NewLogMessage(Message(), Category(), FileToWriteIn(), ComesFromFile(), LogLevel().Level(), ObjectString())
	End Method
	
	Rem
		bbdoc: Checks if two messages are equal.
		about: Checks if @ptoOtherObject and the message which called this method are equal. <br>
		returns: %True if equal, otherwise %false.
	End Rem
	Method Equals:Byte(ptoOtherObject:TObject)
		If(ptoOtherObject = Null) Then Return False
		If(Not TErrorMessage(ptoOtherObject)) Then Return False
		If(ptoOtherObject = Self) Then Return True
		
		Local MSG:TErrorMessage = TErrorMessage(ptoOtherObject)
		
		If(MSG.Category() = Category()) Then
			If(MSG.Message() = Category()) Then
				If(LogLevel().Equals(MSG.LogLevel())) Then
					If(ComesFromFile() = MSG.ComesFromFile()) Then
						If(FileToWriteIn() = MSG.FileToWriteIn()) Then
							If(ObjectString() = MSG.ObjectString()) Then
								Return True
							EndIf
						EndIf
					EndIf
				EndIf
			EndIf
		EndIf
		
		Return False
	End Method
	
	Rem
		bbdoc: Returns the message as a string.
	End Rem
	Method ToString:String()
		Return ("[" + Category() + "]" + " An Error of the level __" + LogLevel().ToString() + "__ occured in the file: " + ComesFromFile() + ..
				". ~n ~t * It says: " + Message() + " ~n ~t -- The following object was used: " + ObjectString())
	End Method
End Type

Rem
	bbdoc: Class for the messages the errorlogger uses.
	about: The #TErrorLogger uses logmessages to log simple messages.
endrem
Type TLogMessage Extends TErrorMessage
	Rem
		bbdoc:Constructor.
		about: Creates a new TErrorMessage. <br>
		@psMessage is the given message. <br>
		@psCategory is the category. <br>
		@psFileToWriteIn is the path of the destination file. <br>
		@piLogLevel is the loglevel. <br>
	End Rem
	Method NewMessage:TLogMessage(psMessage:String, psCategory:String, psFileToWriteIn:String, piLogLevel:Int)
		__SetTimeWhenCreated(CurrentDate() + " - " + CurrentTime())
		__SetCategory(psCategory)
		__SetMessage(psMessage)
		__SetComesFromFile("")
		__SetFileToWriteIn(psFileToWriteIn)
		__SetObjectString("")
		__SetLogLevel(New TLogLevel.NewLogLevel(piLogLevel))
		
		Return Self
	End Method
	
	Rem
		bbdoc:Checks if two messages are equal.
		about: Checks if @ptoOtherObject and the message which called this method are equal. <br>
		returns: %True if equal, otherwise %false.
	End Rem
	Method Equals:Byte(ptoOtherObject:TObject)
		Super.Equals(ptoOtherObject)
	End Method

	Rem
		bbdoc:Copies the object.
	End Rem
	Method Copy:TErrorMessage()
		Return Super.Copy()
	End Method

	Rem
		bbdoc:Returns the message as a string.
	End Rem
	Method ToString:String()
		If(Trim(Category()) = "" And Trim(Message()) = "" And Trim(TimeWhenCreated()) = "") Then Return ""
		Return("[" + Category() + "] " + Message() + " ~t (" + TimeWhenCreated() + ")")
	End Method
End Type

Rem
	bbdoc: Class for the loglevels.
	about: This class is used to specify the seriousness of errors which are logged.
endrem
Type TLogLevel Extends TObject
	Rem
		bbdoc: Everything is logged.
	endrem
	Const LOGLEVEL_DEBUGGING:Int = 1
	Rem
		bbdoc: Normal logging. Not only fatal errors but even not everything.
	endrem
	Const LOGLEVEL_NORMALERRORS:Int = 2
	Rem
		bbdoc: Only fatal errors are logged.
	endrem
	Const LOGLEVEL_FATALONLY:Int = 3
	
	Rem
		bbdoc: The amount of loglevels.
	endrem
	Const LOGLEVEL_COUNT:Int = 3

	Field _iLevel:Int
	
	Rem
		bbdoc: Getter for the Level.
		about: This method returns the level of the Loglevel. <br>
		returns: The level of the loglevel.
	endrem
	Method Level:Int()
		Return _iLevel
	End Method
	
	Rem
		bbdoc: Private method. Do not use!
		about: Sets the level of the loglevel to @piLevel. <br>
		If the given value is too big or too small, then it adjusted.
	endrem
	Method __SetLevel(piLevel:Int)
		If(piLevel < 1) Then piLevel = LOGLEVEL_DEBUGGING
		If(piLevel > LOGLEVEL_COUNT) Then piLevel = LOGLEVEL_COUNT
		_iLevel = piLevel
	End Method
	
	Rem
		bbdoc: Constructor.
		about: Creates a new #TLogLevel and sets its level to @piLevel.
	endrem
	Method NewLogLevel:TLogLevel(piLevel:Int)
		__SetLevel(piLevel)
		
		Return Self
	End Method
	
	Rem
		bbdoc: Copies the object.
		about: Had to been implemented because of #TObject.
	End Rem
	Method Copy:TObject()
		Return New TLogLevel.NewLogLevel(Level())
	End Method
	
	Rem
		bbdoc: Checks if this loglevel and another Object are equal.
		about: Checks if this loglevel and @ptoOtherObject are equal. <br>
		returns: %True if equal, otherwise %false.
	End Rem
	Method Equals:Byte(ptoOtherObject:TObject)
		If(ptoOtherObject = Null) Then Return False
		If(Not TLogLevel(ptoOtherObject)) Then Return False
		If(ptoOtherObject = Self) Then Return True
		
		Local obj:TLogLevel = TLogLevel(ptoOtherObject)
		
		If(obj.Level() = Level()) Then
			Return True
		EndIf
		
		Return False
	End Method

	Rem
		bbdoc: Returns the loglevel as a string.
	End Rem
	Method ToString:String()
		Select Level()
			Case LOGLEVEL_NORMALERRORS
				Return "Loglevel: Normal Errors"
			
			Case LOGLEVEL_FATALONLY
				Return "Loglevel: Fatal Only"
			
			Case LOGLEVEL_DEBUGGING
				Return "Loglevel: Debugging"
		End Select
	End Method
End Type