SuperStrict

Rem
	bbdoc: UnitTestProvider
	about: This module provides a complete unit test environment for code testing.
End Rem
Module siliziumbrain.unittestprovider
ModuleInfo "Version: v1.00"
ModuleInfo "Author: M0rgenstern - Siliziumbrain - Kevin Peter Hoffeld"
ModuleInfo "License: Public Domain"
ModuleInfo "History v1.00: 15 July 2012 - Final Release"

Import siliziumbrain.uberobject
Import siliziumbrain.exceptions
Import siliziumbrain.errorlogger
Import brl.reflection

rem
	bbdoc: An exception for tests.
	about: This exception is thrown everytime a test fails.
endrem
Type TTestFailException Extends TException
	Global IDs:Int = 0
	
	Rem
		bbdoc: Constructor for the exception.
		about: Sets the extern message to @psGivenMessage. <br>
		Can be used like that: Throw new TNullPointerException.NewException.
	endrem
	Method NewException:TTestFailException(psGivenMessage:String)
		__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		__SetGivenMessage(psGivenMessage)
		__SetID(__NextID())
		__SetOwnMessage("A Unit Test failed.")
		__SetOwnType("TestFailException")
		
		Return Self
	End Method
	
	Rem
		bbdoc: Private method. Do not use.
		about: Returns the next ID for a new exception.
	endrem
	Function __NextID:Int()
		IDs:+1
		Return IDs
	End Function
	
	Rem
		bbdoc: Copies the exception.
		about: Creates a deep copy of the exception. <br>
		returns: A deep copy of the exception.
	End Rem
	Method Copy:TTestFailException()
		Local copEx:TTestFailException = New TTestFailException
		 
		copEx.__SetTimeWhenThrown(CurrentDate() + " - " + CurrentTime())
		copEx.__SetGivenMessage(__GivenMessage())
		copEx.__SetID(__NextID())
		copEx.__SetOwnMessage(__OwnMessage())
		copEx.__SetOwnType(__OwnType())
		 
		Return copEx
	End Method
End Type

rem
	bbdoc: Unit Test Provider class.
	about: This class provides a complete unit test environment. <br>
	Classes which have test methods can be registered here. <br>
	Then the methods will automatically be called by using reflection. <br>
	The test results are logged in files.
endrem
Type TUnitTestProvider
	rem
		bbdoc: UnitTests have their own category.
	endrem
	Const CATEGORY:String = "UnitTests"
	
	rem
		bbdoc: Everything has to be logged. So set the log level to a critical level.
	endrem
	Const LOGLEVEL:Int = TLogLevel.LOGLEVEL_DEBUGGING
	
	rem
		bbdoc: The currently used TypeID. (USed for reflection)
	endrem
	Global CurrentTypeID:TTypeId
	
	rem
		bbdoc: List with all registered classes.
	endrem
	Global TypeNameList:TList
	
	rem
		bbdoc: First function to call. Sets up the test environment.
		about: Every class which has test methods has to be registered here. <br>
		Registering a class is simple. The name of every class has to be in @ptlTypeNames. <br>
		A test method has to begin with 'Test__'. Otherwise it will be ignored. Furthermore, a test method has to have a parameter of the type 'Object'.
		Attention: The class name has to be the exact name as in your code, because of the reflection.
	endrem
	Function SetUpUnitTesting(ptlTypeNames:TList)
		TypeNameList = New TList
		For Local name:String = EachIn ptlTypeNames
			TypeNameList.AddLast(name)
		Next
	End Function
	
	rem
		bbdoc: This function starts the unit tests.
		about: The test methods of every registered class is invoked here.
	endrem
	Function StartUnitTesting(psFileToWriteIn:String)
		'Throw an error if there are no registered classes.
		If(TypeNameList.IsEmpty()) Then Throw New TIllegalArgumentException.NewException("Unit Tester was given no classes with tests.")
		
		'Change the loglevel.
		TErrorLogger.Instance().ChangeCurrentErrorLevel(TLogLevel.LOGLEVEL_DEBUGGING)
		
		'For each class in the list.
		For Local name:String = EachIn TypeNameList
			'Get the Type id of the current class.
			CurrentTypeID = TTypeId.ForName(name)
			Local Methods:TList = CurrentTypeID.Methods()
			Local TestMethods:TList = New TList
			
			'Only use methods with 'Test__' at the beginning.
			For Local m:TMethod = EachIn Methods
				If(m.name().StartsWith("Test__")) Then TestMethods.AddLast(m)
			Next
			
			TErrorLogger.Instance().Log_Message("***************************** Methods in Type '" + name + "' are tested. *****************************",  ..
							CATEGORY, psFileToWriteIn, LOGLEVEL)
			TErrorLogger.Instance().Log_NewLine(psFileToWriteIn, LOGLEVEL)
			
			'Invoke every test method in the current class.
			For Local m:TMethod = EachIn TestMethods
				TErrorLogger.Instance().Log_Message("Calling Test Method: " + m.name(), CATEGORY, psFileToWriteIn, LOGLEVEL)
				Local ar:Object[1]
				m.Invoke(CurrentTypeID.NewObject(), ar)
			Next
			
			TErrorLogger.Instance().Log_Message("######################################################################################################",  ..
						CATEGORY, psFileToWriteIn, LOGLEVEL)
			TErrorLogger.Instance().Log_NewLine(psFileToWriteIn, LOGLEVEL)
		Next

		'Wait until the logger wrote everything in the file.
		While(Not TErrorLogger.Instance().HasFinishedWriting())
		Wend
	End Function
	
	rem
		bbdoc: Just logs that a certain method started.
		about: This function can be called when a test method starts. Just makes the log files nicer. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function TestStart(psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		TErrorLogger.Instance().Log_Message("---------------------------- Test '" + psTestName + "' starts. ----------------------------",  ..
						CATEGORY, psFileToWriteIn, LOGLEVEL)
	End Function

	rem
		bbdoc: Just logs that a certain method ended.
		about: This function can be called when a test method ends. Just makes the log files nicer. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function TestEnd(psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		TErrorLogger.Instance().Log_Message("____________________ Test '" + psTestName + "' ended. ____________________",  ..
						CATEGORY, psFileToWriteIn, LOGLEVEL)
		TErrorLogger.Instance().Log_NewLine(psFileToWriteIn, LOGLEVEL)
	End Function
	
	rem
		bbdoc: Checks if the type of two objects is the same.
		about: This function checks if @ptoExpectedObject has the same (dynamic) type as @ptoActualObject. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function AssertTypeAreEqual(poExpectedObject:Object, poActualObject:Object, psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			Local idEx:TTypeId = TTypeId.ForObject(poExpectedObject)
			Local idAc:TTypeId = TTypeId.ForObject(poActualObject)
			If(idEx.Name() <> idAc.Name()) Then
				Throw New TTestFailException.NewException("Types are not equal. Expected was: " + idEx.Name() + " but it was: " + ..
								idAc.ToString() + ". Test failed in : " + psTestName)
			EndIf
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	End Function

	rem
		bbdoc: Checks if two TObjects are equal.
		about: This function checks if @ptoActualObject equals @ptoExpectedObject. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function AssertTObjectAreEqual(ptoExpectedObject:TObject, ptoActualObject:TObject, psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			If(Not ptoExpectedObject.Equals(ptoActualObject)) Then
				Throw New TTestFailException.NewException("TObjects are not equal. Expected was: " + ptoExpectedObject.ToString() + " but it was: " + ..
								ptoActualObject.ToString() + ". Test failed in : " + psTestName)
			EndIf
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	End Function
	
	rem
		bbdoc: Checks if two Objects are equal.
		about: This function checks if @poActualObject equals @poExpectedObject. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function AssertObjectAreEqual(poExpectedObject:Object, poActualObject:Object, psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			If(Not (poExpectedObject = poActualObject)) Then
				Throw New TTestFailException.NewException("Objects are not equal. Expected was: " + poExpectedObject.ToString() + " but it was: " + ..
								poActualObject.ToString() + ". Test failed in : " + psTestName)
			EndIf
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	End Function
	
	rem
		bbdoc: Checks if two Integer values are equal.
		about: This function checks if @piActualValue equals @piExpectedValue. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function AssertIntAreEqual(piExpectedValue:Int, piActualValue:Int, psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			If(piExpectedValue <> piActualValue) Then
				Throw New TTestFailException.NewException("Integers are not equal. Expected was: " + piExpectedValue + " but it was: " + ..
								piActualValue + ". Test failed in : " + psTestName)
			EndIf
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	EndFunction
	
	rem
		bbdoc: Checks if two Byte values are equal.
		about: This function checks if @pbActualValue equals @pbExpectedValue. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function AssertByteAreEqual(pbExpectedValue:Byte, pbActualValue:Byte, psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			If(pbExpectedValue <> pbActualValue) Then
				Throw New TTestFailException.NewException("Bytes are not equal. Expected was: " + pbExpectedValue + " but it was: " + ..
								pbActualValue + ". Test failed in : " + psTestName)
			EndIf
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	EndFunction
	
	rem
		bbdoc: Checks if two Float values are equal.
		about: This function checks if @pfActualValue equals @pfExpectedValue. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function AssertFloatAreEqual(pfExpectedValue:Float, pfActualValue:Float, psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			If(pfExpectedValue <> pfActualValue) Then
				Throw New TTestFailException.NewException("Floats are not equal. Expected was: " + pfExpectedValue + " but it was: " + ..
								pfActualValue + ". Test failed in : " + psTestName)
			EndIf
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	EndFunction
	
	rem
		bbdoc: Checks if two Double values are equal.
		about: This function checks if @pdActualValue equals @pdExpectedValue. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function AssertDoubleAreEqual(pdExpectedValue:Double, pdActualValue:Double, psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			If(pdExpectedValue <> pdActualValue) Then
				Throw New TTestFailException.NewException("Doubles are not equal. Expected was: " + pdExpectedValue + " but it was: " + ..
								pdActualValue + ". Test failed in : " + psTestName)
			EndIf
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	EndFunction
	
	rem
		bbdoc: Checks if an object is not %null.
		about: This function checks if @poObject is not %null. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function AssertNotNull(poObject:Object, psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			If(poObject = Null) Then
				Throw New TTestFailException.NewException("Object is null although it is not supposed to be. Test failed in : " + psTestName)
			EndIf
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	End Function
	
	rem
		bbdoc: Checks if an object is %null%.
		about: This function checks if @poObject is %null. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function AssertNull(poObject:Object, psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			If(poObject <> Null) Then
				Throw New TTestFailException.NewException("Object is not null although it is supposed to be. Test failed in : " + psTestName)
			EndIf
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	End Function
	
	rem
		bbdoc: Checks if an expression is %true.
		about: This function checks if @pbExpression is %true. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function AssertTrue(pbExpression:Byte, psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			If(pbExpression = False) Then
				Throw New TTestFailException.NewException("Expression is false although it is supposed to be true. Test failed in : " + psTestName)
			EndIf
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	End Function
	
	rem
		bbdoc: Checks if an expression is %false.
		about: This function checks if @pbExpression is %false. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function AssertFalse(pbExpression:Byte, psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			If(pbExpression = True) Then
				Throw New TTestFailException.NewException("Expression is false although it is supposed to be true. Test failed in : " + psTestName)
			EndIf
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	End Function
	
	rem
		bbdoc: Just makes the test fail.
		about: Use this function if you want a test to fail. <br>
		@psTestName is the name of the method which calls this function. <br>
		@psComesFromFile is the filename where the test method is in. <br>
		@psFileToWriteIn is the path to the file where the log should be written in.
	endrem
	Function Fail(psTestName:String, psComesFromFile:String, psFileToWriteIn:String)
		Try
			Throw New TTestFailException.NewException("Fail!. Test failed in: " + psTestName)
		Catch ex:TException
			TErrorLogger.Instance().Log_Error(ex.ToString(), CATEGORY, psComesFromFile, psFileToWriteIn, LOGLEVEL)
		End Try
	End Function
End Type